=begin
@author		Ville Kumpulainen

Super class for clustering algorithms. Introduces two required methods for derived classes; step and happy?
=end

require 'dataset'
#require 'metric'

class Clustering
	class Algorithm
		KMEANS						= $running_algorithm_number = 0
		RANDOM_SWAP					= $running_algorithm_number += 1
		SEMIRANDOM_SWAP				= $running_algorithm_number += 1
		RANDOMIZED_LOCAL_SEARCH		= $running_algorithm_number += 1
		SEMIRANDOMIZED_LOCAL_SEARCH	= $running_algorithm_number += 1
		DBSCAN						= $running_algorithm_number += 1
		
		def self.valid? algorithm
			algorithm <= $running_algorithm_number
		end
	end

	attr_reader :data
#	attr_reader :algorithm #@todo: is there a need for this?
	
	def initialize data
		raise "First parameter must be a Dataset object" unless data.kind_of? Dataset
		@data = data
	end
	
	protected :initialize
	
	def self.create data, algorithm, *p
		return case algorithm #(@algorithm = algorithm)
#			when Algorithm::KMEANS
				#@todo
				
			when Algorithm::DBSCAN
				require 'clustering/dbscan'
				DBSCAN.new data, *p
				
			else
				raise "Second parameter must be one of #{Algorithm.constants.map{|c|"Algorithm::#{c}"}.join('/')}" unless Algorithm.valid? algorithm
				raise "Given algorithm #{algorithm} not yet implemented"
		end
	end
	
	#
	# Executes one iteration of chosen algorithm.
	#
	def step
		raise "Please implement step in class #{self.class}"
	end
	
	def iterate #alternative (possibly more recognized) name for iterations
		step
	end
	
	#
	# Informs if further iterations are ineffective. Works as a recommended stopping criterion for a clustering method.
	#
	#@return	True, if it is considered that further iterations result no significant changes on the current clustering. Otherwise false. Any settings affecting decision of happiness should be provided as initial parameters.
	#
	def happy?
		raise "Please implement happy? in class #{self.class}"
	end
end


