require 'sdl'
require 'style'
require 'tools'
require 'color'

class SDL::PixelFormat
	def mapHSL *p
#		puts p.flatten.inspect
		r,g,b = hsl_to_rgb p.flatten
#		puts [r,g,b].inspect
		mapRGB r,g,b
	end
end

class DrawableSDLSurface
	include Style
	include Tools
	
	@screen = nil
	
	@color = nil
	@mapped_color = nil
	@black = nil
	@white = nil
	@stroke_width = 1
	#TODO: stroke_width etc.
	
	attr_reader :width, :height, :tool
	
	def initialize width = 800, height = 600
		@width, @height = width, height
		
		SDL.init SDL::INIT_VIDEO
		@screen = SDL::set_video_mode width, height, 24, SDL::SWSURFACE
		@screen.lock
		@black = @screen.format.mapRGB 0, 0, 0
		@white = @screen.format.mapRGB 255, 255, 255
		@screen.fill_rect 0, 0, width, height, @white
		
		init_tools
	end
	
	def init_tools
		#randomize initial color
		@color = Array.new(3).map { rand(120) + 120 }
		@mapped_color = @screen.format.mapHSL @color

		@stroke_width = 1
	end
	
	def set_tool tool #TODO: unnecessary in this particular implementation?
		case tool
			when TOOL_O
				
				
			when TOOL_X #drawn by line
				
			
			when TOOL_LINE #or black?
				
			
			when TOOL_TEXT
				
			
			else
				raise "Unidentified tool #{tool}!"
		end
		
		@tool = tool
	end
	
	private :set_tool

	def next_color #as hsl
		@color = [((@color[0] + 77) % 360), (110 + rand(127)), (128 + rand(70))]
		set_hsl_color @color
		#@tool = TOOL_NONE #refresh color next time we're drawing something..
		#@todo in the future call set_color (or remove this method)
	end
	
	def set_color_of i, max
		set_hsl_color 360 * i / max, 190, 190
		return 360 * i / max
	end
	
	def set_color i
		#@todo: invent good variation to s and l in situations where the color wheel is sliced in millions of pieces (or even thousands)
		raise "Given parameter must be a non-negative Fixnum" unless i.class == Fixnum || i < 0
		split_point = 256
		h = s = 0.0
#=begin		
		k = i / split_point
		j = 1
		while k > 0 do
			if k % 2 == 1
				s += 80.0 / (2 ** j)
			end
			k /= 2
			j += 1
		end
		k = i % split_point
#=end	
#		k = i
		j = 1
		while k > 0 do
			if k % 2 == 1
				h += 360.0 / (2 ** j)
			end
			k /= 2
			j += 1
		end
#		set_hsl_color h.to_i, 170 + Math.sin(h*0.0413) * 85, 140 + Math.cos(h*0.0173) * 70
		set_hsl_color h.to_i, 120 + s.to_i, 192 #140 + Math.cos(h*0.0173) * 70
		return h
	end
	
	def set_hsl_color h, s, l
		p = [h,s,l]
#		raise "Three values (hue, saturation and lightness) should be provided!" if p.length != 3
		raise "Hue should be in 0..359 and both saturation and lightness in 0..255!" unless p.zip([0..359]+[0..256]*2).all? {|i|i[1].include? i[0]}
		@mapped_color = @screen.format.mapHSL p
	end

	#method that respects @stroke_width
	def _line x1, y1, x2, y2, color
		dx = x2 - x1
		dy = y2 - y1
		d = (dx.abs < dy.abs ? dy.abs : dx.abs) + 1
		d.to_i.times { |i|
			x = x1 + dx * i / d
			y = y1 + dy * i / d
			@screen.draw_rect x, y, @stroke_width, @stroke_width, color
		}
	end
	
	private :_line

	def clear
		@screen.fill_rect 0, 0, width, height, @white
	end

	def plot *x
		plot_x(*x)
	end
	
	def plot_o x, y
		set_tool TOOL_O unless @tool == TOOL_O
		#@surface.ellipse x*@width, y*@height, PLOT_RADIUS, PLOT_RADIUS, 0, 360
		@screen.draw_circle x*@width, y*@height, PLOT_RADIUS, @black, true
		@screen.draw_circle x*@width, y*@height, PLOT_RADIUS - @stroke_width, @mapped_color, true
	end
	
	def plot_x x, y
		#TODO: shadow
		set_tool TOOL_X unless @tool == TOOL_X
=begin
		@stroke_width = 3
		@screen.draw_line x*@width - PLOT_RADIUS + 1, y*@height - PLOT_RADIUS + 1, x*@width + PLOT_RADIUS - 1, y*@height + PLOT_RADIUS - 1, @black
		@screen.draw_line x*@width - PLOT_RADIUS + 1, y*@height + PLOT_RADIUS - 1, x*@width + PLOT_RADIUS - 1, y*@height - PLOT_RADIUS + 1, @black
		@stroke_width = 1
		@screen.draw_line x*@width - PLOT_RADIUS + 1, y*@height - PLOT_RADIUS + 1, x*@width + PLOT_RADIUS - 1, y*@height + PLOT_RADIUS - 1, @mapped_color
		@screen.draw_line x*@width - PLOT_RADIUS + 1, y*@height + PLOT_RADIUS - 1, x*@width + PLOT_RADIUS - 1, y*@height - PLOT_RADIUS + 1, @mapped_color
=end
		@stroke_width = 3
		_line x*@width - PLOT_RADIUS + 1, y*@height - PLOT_RADIUS + 1, x*@width + PLOT_RADIUS - 1, y*@height + PLOT_RADIUS - 1, @black
		_line x*@width - PLOT_RADIUS + 1, y*@height + PLOT_RADIUS - 1, x*@width + PLOT_RADIUS - 1, y*@height - PLOT_RADIUS + 1, @black
		@stroke_width = 1
		_line x*@width - PLOT_RADIUS + 1, y*@height - PLOT_RADIUS + 1, x*@width + PLOT_RADIUS - 1, y*@height + PLOT_RADIUS - 1, @mapped_color
		_line x*@width - PLOT_RADIUS + 1, y*@height + PLOT_RADIUS - 1, x*@width + PLOT_RADIUS - 1, y*@height - PLOT_RADIUS + 1, @mapped_color
	end
	
	def line x1, y1, x2, y2
		set_tool TOOL_LINE unless @tool == TOOL_LINE
		#@screen.draw_line x1*@width, y1*@height, x2*@width, y2*@height, @mapped_color
		_line x1*@width, y1*@height, x2*@width, y2*@height, @mapped_color
	end
	
	def text x, y, msg
		set_tool TOOL_TEXT unless @tool == TOOL_TEXT
		#TODO..
		#@surface.text x*@width, y*@height + 9, msg #TODO: fix vertical alignment properly!
	end
	
	def display
		@screen.flip
	end
	
	def write file
		@screen.save_bmp file
	end
end

